/*
 *  Copyright (C) 2002 Tomasz Kojm <zolw@konarski.edu.pl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>

#include "options.h"
#include "others.h"
#include "shared.h"
#include "manager.h"
#include "defaults.h"

void help(void);

/* this local macro takes care about freeing memory at exit */
#define mexit(i)    if(opt) free_opt(opt);			    \
		    mprintf("*Memory freed. Exit code: %d\n", i);   \
		    exit(i);

void clamscan(struct optstruct *opt)
{
	int ds, dms, ret;
	struct timeval t1, t2;
	struct timezone tz;
	time_t starttime;


    /* initialize some important variables */

    if(optc(opt, 'v')) mprintf_verbose = 1;
    else mprintf_verbose = 0;

    if(optl(opt, "quiet")) mprintf_quiet = 1;
    else mprintf_quiet = 0;

    if(optl(opt, "stdout")) mprintf_stdout = 1;
    else mprintf_stdout = 0;

    if(optc(opt, 'V')) {
	mprintf("ClamScan, version "VERSION"\n");
	mexit(0);
    }

    if(optc(opt, 'h')) {
	free_opt(opt);
    	help();
    }

    /* check other options */

    if(optc(opt, 'r')) recursion = 1;
    else recursion = 0;

    if(optl(opt, "one-virus")) onevir = 1;
    else onevir = 0;

    /* initialize logger */

    if(optl(opt, "log-verbose")) logverbose = 1;
    else logverbose = 0;

    if(optc(opt, 'l')) {
	logfile = getargc(opt, 'l');
	if(logg("--------------------------------------\n")) {
	    mprintf("!Problem with internal logger.\n");
	    mexit(1);
	}
    } else 
	logfile = NULL;


    time(&starttime);
    /* ctime() do \n, but I need once more */
    logg("Scan started: %s\n", ctime(&starttime));

    memset(&claminfo, 0, sizeof(struct s_info));

    gettimeofday(&t1, &tz);
    ret = scanmanager(opt);

    if(!optl(opt, "disable-summary")) {
	gettimeofday(&t2, &tz);
	ds = t2.tv_sec - t1.tv_sec;
	dms = t2.tv_usec - t1.tv_usec;
	ds -= (dms < 0) ? (1):(0);
	dms += (dms < 0) ? (1000000):(0);
	mprintf("\n----------- SCAN SUMMARY -----------\n");
	    logg("\n-- summary --\n");
	mprintf("Known viruses: %d\n", claminfo.signs);
	    logg("Known viruses: %d\n", claminfo.signs);
	mprintf("Scanned directories: %d\n", claminfo.dirs);
	    logg("Scanned directories: %d\n", claminfo.dirs);
	mprintf("Scanned files: %d\n", claminfo.files);
	    logg("Scanned files: %d\n", claminfo.files);
	mprintf("Infected files: %d\n", claminfo.ifiles);
	    logg("Infected files: %d\n", claminfo.ifiles);
	mprintf("Found viruses: %d\n", claminfo.viruses);
	    logg("Found viruses: %d\n", claminfo.viruses);
	//mprintf("Total errors: %d\n", claminfo.errors);
	if(claminfo.threads) {
	    mprintf("Used threads: %d\n", claminfo.threads);
		logg("Used threads: %d\n", claminfo.threads);
	}
	mprintf("I/O buffer size: %d bytes\n", BUFFSIZE);
	    logg("I/O buffer size: %d bytes\n", BUFFSIZE);
	mprintf("Time: %d.%3.3d sec\n", ds, dms/1000);
	    logg("Time: %d.%3.3d sec\n\n", ds, dms/1000);
    }

    mexit(ret);
}

void help(void)
{

    mprintf_stdout = 1;

    mprintf("
		   Clam Antivirus: The Scanner (ClamScan)  "VERSION"
		   (c) 2002 Tomasz Kojm <zolw@konarski.edu.pl>
	  
    --help		    -h		show help
    --version		    -V		print version number and exit
    --verbose		    -v		be verbose
    --quiet				be quiet, output only error messages
    --stdout				write to stdout instead of stderr
					(this help is always written to stdout)
    --force				try to ignore some errors

    --tempdir DIRECTORY			create temporary files in DIRECTORY
    --database	FILE	    -d FILE	read virus database from FILE
    --log  FILE	            -l FILE	save scan report in FILE
    --log-verbose			save additional informations
    --whole-file	    -w		scan whole file
    --recursive		    -r		scan directories recursively
    --one-virus				stop scanning a file if virus found
    --disable-summary			disable summary at end of scanning

    --max-space #n			extract first #n kilobytes only
    --max-files #n			extract first #n files only
    --unzip[=FULLPATH]			enable support for .zip files
    --unrar[=FULLPATH]			enable support for .rar files
    --unace[=FULLPATH]			enable support for .ace files
    --unarj[=FULLPATH]			enable support for .arj files
    --zoo[=FULLPATH]			enable support for .zoo files
    --lha[=FULLPATH]			enable support for .lha files
    --jar[=FULLPATH]			enable support for .jar files
    --tar[=FULLPATH]			enable support for .tar files
    --tgz[=FULLPATH]			enable support for .tar.gz, .tgz files\n\n");

#ifdef THREADS
    mprintf("    --threads #n			use n threads\n");
#endif THREADS

    exit(0);
}
